`baseline_estimate` <-
function(data, subtract=TRUE, method="none", path="") {
  METHODS <- c("none", "file", "lowess", "smooth", "smooth_spline", 
               "bot_spline", "smooth_bot_spline", "PROcess")
  if (is.na(pmatch(method, METHODS))) 
    stop("invalid baseline estimation method")

  if (method == "file") {
    print("Loading files containing baseline corrected spectra.")
    classnames <- unique(data$conf$classlabels)

    # Check if our original data was log-transformed.
    if (data$conf[["log"]]) { do_log = TRUE; } else { do_log = FALSE; }
    base_file <- load_data(classnames=classnames, path=path, do_log=do_log) 

    print("Done.");
  } else if (method == "PROcess") {
    library(PROcess)
  }

  num.samples <- ncol(data$int);

  base <- matrix(0, nrow=nrow(data$int), ncol=num.samples)

  for (i in 1:num.samples) {
    print(paste("Estimating baseline for", colnames(data$mz)[i]));

    mz_values   <- data$mz[,i]; 
    intensities <- data$int[,i]; 

    base[,i] <- switch(method, 
      none              = 0,
      file              = intensities - base_file$int[,i],
      lowess            = lowess(mz_values, intensities, f=0.01, delta=100)$y,
      smooth            = supsmu(mz_values, intensities, span=0.01)$y,
      smooth_spline     = smooth.spline(mz_values, intensities)$y,
      bot_spline        = { neg_peak_indexes <- get_peaks_in_spectrum(intensities, negative=TRUE, iter=3);
                            spline_func <- splinefun(mz_values[neg_peak_indexes], intensities[neg_peak_indexes]);
                            spline_func(mz_values); },
      smooth_bot_spline = { neg_peak_indexes <- get_peaks_in_spectrum(intensities, negative=TRUE, iter=3);
                            spline_func <- splinefun(mz_values[neg_peak_indexes], smooth(intensities[neg_peak_indexes]));
                            spline_func(mz_values); },
      PROcess           = intensities - (bslnoff(cbind(mz_values, intensities), method="loess")[,2])
    );

    if (subtract == TRUE) {
      data$int[,i] <- intensities - base[,i];
    }
  }

  data[["base"]] <- data.frame(base);
  colnames(data$base) <- colnames(data$mz);

  data$conf[["baseline_subtract"]] <- subtract;
  data$conf[["baseline_method"]]   <- method;

  data
}

