`calc_offset_dev` <-
function(intensities, offset_method="mean", dev_method="sd", 
                           scale="global", width=1000) {
  library(caTools)

  offset_methods <- c("none", "mean", "median", "min")
  if (is.na(pmatch(offset_method, offset_methods))) 
    stop("invalid method for calculating the offset")

  dev_methods   <- c("none", "mean", "median", "range", "quant", "sd", "mad")
  if (is.na(pmatch(dev_method, dev_methods))) 
    stop("invalid method for calculating the deviation")

  if ((width %% 2) == 0) { width <- width+1; }

  if (scale == "global") { ## GLOBAL ESTIMATION
    offset <- switch(offset_method, 
      none        = 0,
      mean        = mean(intensities,               na.rm=TRUE),
      median      = median(intensities,             na.rm=TRUE),
      min         = min(intensities,                na.rm=TRUE)
    );

    deviation <- switch(dev_method,
      none        = 0,
      mean        = mean(intensities,               na.rm=TRUE),
      median      = median(intensities,             na.rm=TRUE),
      range       = diff(range(intensities,         na.rm=TRUE)),
      quant       = diff(quantile(intensities,      na.rm=TRUE, probs=c(0.25,0.75))),
      sd          = sd(intensities,                 na.rm=TRUE),
      mad         = mad(intensities,                na.rm=TRUE)
    );
  } else { ## LOCAL ESTIMATION
    offset <- switch(offset_method, 
      none        = rep(0, length(intensities)),
      mean        = runmean(intensities,                      k=width),
      median      = runmed(intensities,                       k=width,                       endrule="constant"),
      min         = runmin(intensities,                       k=width,                       endrule="constant")
    );

    deviation <- switch(dev_method,
      none        = rep(0, length(intensities)),
      mean        = runmean(intensities,                      k=width),
      median      = runmed(intensities,                       k=width,                       endrule="constant"),
      range       = as.vector(diff(t(runquantile(intensities, k=width, probs=c(0,1),         endrule="constant")))),
      quant       = as.vector(diff(t(runquantile(intensities, k=width, probs=c(0.25,0.75),   endrule="constant")))),
      sd          = runsd(intensities,                        k=width),
      mad         = runmad(intensities,                       k=width, constant=1,           endrule="constant")
    );
  }

  list(offset=offset,deviation=deviation);
}

