`get_cluster_data` <-
function(cluster_list, data, perc_width=0.3) {
  print("Obtaining data for all spectra per cluster")
  num_samples <- ncol(data$mz)
  half_max_peak_width <- log10(1 + (perc_width / 100))/2;

  # Obtain means of log10 M/Z-values per cluster
  clust_means <- unlist(lapply(cluster_list, function(x) { log10(mean(x$mz, na.rm=TRUE)) }))

  for (i in 1:num_samples) {
    print(paste("Obtaining cluster data for", colnames(data$mz)[i]))
    mz_values      <- data$mz[,i];
    intensities    <- data$int[,i];
    mz_values_log  <- log10(mz_values); 

    for (j in 1:length(cluster_list)) {
      cluster <- cluster_list[[j]];
      if (!(i %in% cluster[,"spectrum"])) {
        min <- which.min(abs(mz_values_log - (clust_means[j] - half_max_peak_width)))
        max <- which.min(abs(mz_values_log - (clust_means[j] + half_max_peak_width)))
        interval <- min:max;

        hit <- which.max(intensities[interval])[1]
        cluster <- rbind(cluster, c(mz_values[interval[hit]], intensities[interval[hit]], i))
      }
      #cluster$int[which(cluster$int < 0)] <- 0; # Set the minimum value to zero.
      #cluster$int[which(is.na(cluster$int))] <- 0; # Set unknown values to zero.
      cluster$int[which(cluster$int == Inf)] <- NA; # Set infinite values to NA.
      cluster_list[[j]] <- sort.data.frame(cluster, key="spectrum");
    }
  }

  for (j in 1:length(cluster_list)) {
    cluster <- cluster_list[[j]];

    hits <- which(is.na(cluster$mz));
    cluster$mz[hits] <- mean(cluster$mz[-hits], na.rm=TRUE);
    hits <- which(is.na(cluster$int));
    cluster$int[hits] <- mean(cluster$int[-hits], na.rm=TRUE);

    cluster_list[[j]] <- cluster;
  }

  cluster_list
}

