`get_clusters` <-
function(peak_data, perc_width=0.3, merge_clusters=FALSE) {
  names(peak_data) <- c("mz", "int", "spectrum")
  sorted_peaks     <- sort.data.frame(peak_data, key="mz")

  # Log transform data so that we can work with percentages.
  sorted_peaks_log <- log10(sorted_peaks$mz); 
  uniq_sorted_peaks_log <- unique(sorted_peaks_log)

  if (length(sorted_peaks_log) < 2) { return(); }
  max_peak_width   <- log10(1 + (perc_width / 100));

  # We should not skip any values, which is why we factorise.
  ord <- as.numeric(as.factor(rank(sorted_peaks_log, ties.method="min")))

  print(paste("Finding clusters in", length(sorted_peaks_log), "peaks using", 
        length(uniq_sorted_peaks_log), "unique peak positions"))

  all_found_clusters = c()
  i <- 1;
  # Just in case we have a really tiny dataset
  j <- min(1000, length(uniq_sorted_peaks_log)); 
  while(j <= length(uniq_sorted_peaks_log)) {
    if (length(uniq_sorted_peaks_log) == 0) { break; }
    print(paste("Processing window", i, "to", j, "..."))

    #dendro <- hclust(dist(sorted_peaks_log[i:j]), method="complete")
    dendro <- hclust1d(uniq_sorted_peaks_log[i:j])
    found_clusters <- cutree(dendro, h=max_peak_width)

    # This can only be processed if more than one cluster was found.
    # This is because we do not take the last cluster into account,
    # because it may need to be merged with data in the next window.
    max_found_clusters <- max(found_clusters)
    if (max_found_clusters > 1) { 
      if (length(all_found_clusters) > 0) {
        max_all_found_clusters <- max(all_found_clusters)
      } else {
        max_all_found_clusters <- 0
      }

      all_found_clusters <- c(all_found_clusters, 
                                found_clusters[which(found_clusters < 
                                  max_found_clusters)] + 
                                max_all_found_clusters)
    } 

    i <- min((which(found_clusters == max(found_clusters)))+(i-1));
    if (j != length(uniq_sorted_peaks_log)) { 
      j <- min(j+1000, length(uniq_sorted_peaks_log));
    } else {
      j <- j+1; 
    }
  }

  if (merge_clusters == TRUE) {
    all_found_clusters <- merge_clusters(all_found_clusters, uniq_sorted_peaks_log)
  }

  # This is a try-out; much faster but perhaps less easy to handle.
  #clusters <- cbind(sorted_peaks, all_found_clusters[ord])

  ## Rebuild original peak data list into the new peak clusters.
  cluster_list <- as.list(unique(all_found_clusters));
  # The 'ord' part is crucial and is used to map the unique peaks back to the original 'peak_data'
  cluster_list <- lapply(cluster_list, function(x) 
                           sorted_peaks[which(all_found_clusters[ord] == x), ])

  cluster_list
}

