`hclust1d` <-
function(x, sortx=TRUE) {
  if(sortx) {
    ord <- order(x)
    x <- sort(x)
  } else {
    ord <- 1:length(x)
  }

  # We assume 'x' to be non-redundant.
  if (length(x) != length(unique(x))) {
    stop("Error during clustering: 'x' not non-redundant")
  }

  inds <- 1:length(x);
  label <- -(1:length(x));
  label <- label[ord];
  height <- c()
  merge <- matrix(0,length(x)-1,2)

  ks <- c()
  if (length(x) > 1) { ks <- 1:(length(x)-1) }

  for(k in ks) {
    dist <- diff(x[inds]);
    hits <- which(dist >= 0); # Positive valued entries; these are unclustered entries.
    j <- hits[which.min(dist[hits])]

    if (j != (length(inds)-1) && (dist[j+1] < 0)) {
      # Merge point with cluster on right
      inds <- inds[-(j+2)]
    }

    if (j != 1 && (dist[j-1] < 0)) {
      # Merge point with cluster on left
      inds <- inds[-(j-1)]
      j <- j-1; # Removed one before, so update j
    }

    height[k] <- diff(x[inds[c(j,j+1)]]);

    # Switch elements in original data.
    # This makes it possible to represent any cluster by two numbers;
    # its minimum and maximum.
    x[inds[c(j,j+1)]] <- x[inds[c(j+1,j)]]

    merge[k,] <- label[inds[c(j,j+1)]]
    label[inds[c(j,j+1)]] <- k  # Register new cluster
  }

  hc <- list(merge=merge,height=height,order=ord,labels=NULL,
             method="complete",call=match.call(),dist.method="euclidean")
  class(hc) <- "hclust"

  hc
}

