`load_data` <-
function(classnames=c(), path="", long_samplelabels=FALSE, do_log=FALSE) {
  if (length(classnames) == 0) 
    stop("Please provide at least one classname")

  data <- list()
  data[["mz"]]  <- data.frame()
  data[["int"]] <- data.frame()

  if (path == "") {
    path <- getwd();
  }

  classlabels  <- c();
  samplelabels <- c();

  for (classname in classnames) {
    list.of.files <- dir(path = paste(path, classname, sep="/"), 
                         pattern = "\\.csv")
    if (long_samplelabels) 
      samplelabels  <- c(samplelabels, paste(classname, "-", gsub("\\.csv", "", list.of.files)))
    else 
      samplelabels  <- c(samplelabels, gsub("\\.csv", "", list.of.files))
    classlabels   <- c(classlabels, rep(classname, length(list.of.files)))

    for(file.name in list.of.files) {
      print(paste("Loading", file.name));

      file <- read.table(paste(path, classname, file.name, sep="/"), 
                         header = TRUE, sep = ",")

      mz_values   <- file[,1];
      if (do_log) {
        intensities <- log(file[,2]);
      } else {
        intensities <- file[,2];
      }

      # Remove duplicate mz_value entries.
      dupes <- which(duplicated(mz_values))
      if (length(dupes) > 0) {
        mz_values <- mz_values[-dupes]
        intensities <- intensities[-dupes]
      }

      if (ncol(data[["mz"]]) > 0) {
        # This will take care of dimension mismatches
        if (length(mz_values) < nrow(data[["mz"]])) {
          print("Padding file")
          pad_length <- nrow(data[["mz"]]) - length(mz_values)
          mz_values   <- c(mz_values,   rep(NA, pad_length));
          intensities <- c(intensities, rep(NA, pad_length));
        }

        if (length(mz_values) > nrow(data[["mz"]])) {
          print("Padding data")
          pad_length <- length(mz_values) - nrow(data[["mz"]])
          padding <- matrix(NA, nrow=pad_length, ncol=ncol(data[["mz"]]));
          colnames(padding) <- colnames(data[["mz"]])
          data[["mz"]]  <- rbind(data[["mz"]],  padding);
          colnames(padding) <- colnames(data[["int"]])
          data[["int"]] <- rbind(data[["int"]], padding);
        }
      }

      if (ncol(data[["mz"]]) == 0) {
        data[["mz"]]  <- data.frame(mz_values);
        data[["int"]] <- data.frame(intensities);
      } else {
        data[["mz"]]  <- cbind(data[["mz"]],  mz_values);
        data[["int"]] <- cbind(data[["int"]], intensities);
      }
    }
  }

  colnames(data[["mz"]])  <- samplelabels
  colnames(data[["int"]]) <- colnames(data[["mz"]])

  data[["conf"]] <- list()
  data[["conf"]][["classlabels"]] <- classlabels
  data[["conf"]][["log"]] <- do_log

  data
}

