`normalisation` <-
function(data, offset="mean", deviation="sd", scale="local", min_mz=-Inf, min_int=-Inf) {
  num.samples <- ncol(data$mz);

  for (i in 1:num.samples) {
    #print(paste("Normalising", colnames(data$mz)[i]));
    mz_values   <- data$mz[,i]; 
    intensities <- data$int[,i]; 

    if (scale == "global") {
      include <- which(mz_values >= min_mz & intensities >= min_int);
    } else {
      include <- 1:length(mz_values);
    }

    noisedev <- calc_offset_dev(intensities[include], offset_method=offset, 
                                dev_method=deviation, scale=scale)

    if (offset != "none" | deviation != "none") {
      intensities <- (intensities - noisedev$offset) / noisedev$deviation;
      hits <- which( intensities == Inf | intensities == -Inf | 
                     is.na(intensities) | intensities == NaN)

      if (length(hits) > 0) {
        print("Imputing missing values");
        lm.res <- lm(y ~ x, data = data.frame(y=intensities[-hits], x=mz_values[-hits]))
        intensities[hits] <- predict.lm(lm.res, data.frame(x=mz_values[hits]))
      }
      data$int[,i] <- intensities;
      #data$int[is.na(data$int[,i]), i] <- 0; # Remove NaN entries due to division of 0 by 0
    }
  }
  
  data$conf[["normalisation_offset_method"]] <- offset
  data$conf[["normalisation_dev_method"]]   <- deviation
  data$conf[["normalisation_scale"]]        <- scale
  data$conf[["normalisation_min_mz"]]       <- min_mz
  data$conf[["normalisation_min_int"]]      <- min_int

  data
}

