`plot_clusters` <-
function(cluster_data, data, perc_width=0.1, to_exclude=c(), 
                          single_class=0, pdf_file="", boxplots=TRUE) {
  library(aplpack)

  if (pdf_file != "") {
    pdf(file = pdf_file, version="1.4")
  } else {
    x11()
  }

  classlabels <- data$conf$classlabels;
  classes <- unique(classlabels);

  #layout_mat <- matrix(c(1:2), 1*2, 1);
  #nf <- layout(layout_mat, widths=c(1), heights=c(1,2))
  layout_mat <- rbind(matrix(c(1:6), 2, 3), matrix(c(7:12), 2, 3));
  nf <- layout(layout_mat, widths=c(1, 1, 1), heights=c(1,2,1,2))
  #layout.show(nf) # For debugging purposes.

  for (i in 1:length(cluster_data)) {
    if (!(i %in% to_exclude) & length(cluster_data[[i]]) > 0) {
      cluster <- cluster_data[[i]];
      print(paste("Plotting peak cluster", i))

      # Obtain feature details
      median_peak_mz <- median(cluster[,"mz"], na.rm=TRUE);
      min_x_value    <- min(   cluster[,"mz"], na.rm=TRUE);
      max_x_value    <- max(   cluster[,"mz"], na.rm=TRUE);

      #ext_feature_details <- extend_feature_region(min_x_value, max_x_value, median_peak_mz, extension=5)
      #min_x_value_ext     <- ext_feature_details$ext_min
      #max_x_value_ext     <- ext_feature_details$ext_max
      min_x_value_ext     <- min_x_value
      max_x_value_ext     <- max_x_value

      size_x_interval     <- max_x_value - min_x_value
      size_x_interval_ext <- max_x_value_ext - min_x_value_ext

      # Used for positioning of the vertical boxplot (for the single class case)
      extra_space     <- size_x_interval*0.1 
      extra_space_ext <- size_x_interval_ext*0.1

      # Perhaps this should be done better by not using data$mz[,1]
      lines_indexes <- which( data$mz[,1] < max_x_value_ext & 
                              data$mz[,1] > min_x_value_ext ) 
      
      # This is needed for peak clusters with extremely small intervals
      if (length(lines_indexes) == 0) { 
        print(paste("Skipping cluster", i, "(interval too small)"))
        next 
      }

      lines_indexes_ext <- c( min(lines_indexes, na.rm=TRUE) - c(3:1), lines_indexes, 
                              max(lines_indexes, na.rm=TRUE) + c(1:3) )

      # This is needed for peak clusters close to the start and end of the spectrum
      valid_indexes <- which(lines_indexes_ext > 0 & lines_indexes_ext < length(data$mz[,1]))
      lines_indexes_ext <- lines_indexes_ext[valid_indexes]

      min_y_value <- min(data$int[lines_indexes_ext, ], na.rm=TRUE)
      max_y_value <- max(data$int[lines_indexes_ext, ], na.rm=TRUE)

      ############################################################################

      colour_list <- list( c("#ffaacc", "#ff3355", "#ff7799", "#880000"),
                           c("#aaccff", "#3355ff", "#7799ff", "#000088"),
                           c("#ccffaa", "#55ff33", "#99ff77", "#008800") )

      ############################################################################
      # In order to most efficiently use the available space, only
      # plot between the minimum and maximum values.

      if (single_class == 0) {
        extra_space_ext=0;
      }

      par(mar = c(0,2,3,2))
      plot(1, 1, type="n", xaxt="n", yaxt="n", 
           ylim=c(min_y_value, max_y_value), #main=paste("Cluster", i),
           xlim=c(min_x_value_ext, max_x_value_ext+extra_space_ext)) # Reserve space for vertical boxplot.
      axis(2, at=c(min_y_value, max_y_value), 
           labels=round(c(min_y_value, max_y_value)))
      rug(cluster[,"int"], side=2, ticksize=-0.03)
      # This doesn't display both values, because they are too close together.
      #axis(3, at=c(min_x_value, max_x_value)) 
      # The following is a hack to correct that behaviour.
      axis(3, at=c(min_x_value, max_x_value), labels=FALSE)
      axis(3, at=c(min_x_value*0.996, max_x_value*1.004), 
           labels=round(c(min_x_value, max_x_value), 2), tick=FALSE)
      rug(cluster[,"mz"], side=3, ticksize=-0.03)

      if (single_class == 0) {
        # Plot individual (sub)spectra, one colour for each class.
        for (j in 1:length(classes)) {
          colours <- colour_list[[(j %% length(colour_list))+1]];
          tmp_mz  <- data$mz[lines_indexes_ext,  which(classlabels == classes[j])]
          tmp_int <- data$int[lines_indexes_ext, which(classlabels == classes[j])]
          for (k in 1:ncol(tmp_mz)) {
            lines(tmp_mz[,k], tmp_int[,k], col=colours[3])
          }
        }
      } else {
        # Plot individual (sub)spectra.
        for (j in which(classlabels == classes[single_class])) {
          lines(data$mz[lines_indexes_ext, j], data$int[lines_indexes_ext, j], col=j)
        }

      }

      # Plot vertical lines for median and cluster/centroid limits.
      abline(v=median_peak_mz, col="red")
      abline(v=min_x_value,    col="red", lty=2)
      abline(v=max_x_value,    col="red", lty=2)

      ############################################################################

      par(mar = c(2.2,2,0.2,2))

      if (single_class == 0) {
        plot(1, 1, type="n", ylim=c(0, max_y_value*1.1),
             xlim=c(min_x_value, max_x_value)) # Reserve space for vertical boxplot.
  
        for (j in 1:length(classes)) {
          colours <- colour_list[[(j %% length(colour_list))+1]];
          tmp_data <- cluster[which(cluster$spectrum %in% which(classlabels == classes[j])), c("mz","int")]
          if (nrow(tmp_data) > 1) {
            bagplot(tmp_data, col.loophull=colours[1], col.looppoints=colours[2], 
                    col.baghull=colours[3], col.bagpoints=colours[4], 
                    xlim=c(min_x_value, max_x_value), ylim=c(0,max_y_value*1.1), 
                    xlab="", ylab="", add=TRUE, transparency=TRUE)
          }
        }
      } else {
        tmp_data <- cluster[which(cluster$spectrum %in% which(classlabels == classes[single_class])), c("mz","int")]
        bagplot(tmp_data, xlim=c(min_x_value, max_x_value+extra_space), # Reserve space for vertical boxplot
                ylim=c(0,max_y_value*1.1), xlab="", ylab="")

        if (boxplots) { # This is for not plotting any boxplots when we are trying to work with S/N ratios
          boxplot(tmp_data, add=TRUE, at=max_y_value*1.1, horizontal=TRUE)
          boxplot(tmp_data, add=TRUE, at=max_x_value+(0.75*extra_space))
        }
      }

      abline(v=min_x_value, col="red", lty=2)
      abline(v=max_x_value, col="red", lty=2)
    }
  }

  if (pdf_file != "") {
    dev.off()
  }
}

